__author__ = 'Steven C. Porter'
import datetime
import urllib
import urllib2
import arcpy
import itertools
import os
import sys

#temp filter
#year_filter = range(1998, datetime.datetime.now().year)
year_filter = range(1990, datetime.datetime.now().year)


version = sys.version.split()[0].split('.')
if version[1]== 7 and version[2] <9:
    import ssl
    ssl._create_default_https_context = ssl._create_unverified_context

def check_folder(path):
    if not os.path.exists(path):
        os.makedirs(path)

def arcpyDeleteIfExists(path):
    if arcpy.Exists(path):
        arcpy.Delete_management(path)

def getSoilPolyGML(extent, save_file):
    wfs_url = 'http://sdmdataaccess.sc.egov.usda.gov/Spatial/SDMWGS84Geographic.wfs'
    payload = {
        'SERVICE':'WFS'
        ,'VERSION':'1.1.0'
        ,'REQUEST':'GetFeature'
        ,'TYPENAME':'MapunitPoly'
        ,'OUTPUTFORMAT':'gml2'
        ,'SRSNAME':'EPSG:4326'
        ,'FILTER': '<Filter><BBOX><PropertyName>Geometry</PropertyName><Box srsName=\'EPSG:4326\'><coordinates>'+ '%f,%f %f,%f' %tuple(extent)+'</coordinates></Box></BBOX></Filter>'
    }
    data = urllib.urlencode(payload)
    full_url = wfs_url + '?' + data
    print(full_url)
    response = urllib2.urlopen(full_url)
    gml_data = response.read()
    with open(save_file, 'wb') as output:
        output.write(gml_data)
    return None

def buffer_extent_by_perc(extent, percent):
    """

    :param extent: arcpy.Extent object
    :param percent: percent to increase (or decrease if negative) the extent
    :return: arcpy.Extent
    """
    dx = abs(extent.XMax - extent.XMin) * percent
    dy = abs(extent.YMax - extent.YMin) * percent
    return arcpy.Extent(extent.XMin - dx, extent.YMin - dy, extent.XMax + dx, extent.YMax + dy)


def extent_to_envelope(extent):
    return ' '.join([str(extent.XMin), str(extent.YMin), str(extent.XMax), str(extent.YMax)])


def progress_message(message):
    '''
    Adds an arcpy message and sets the message of the progress bar
    :param message: Text message to display
    :return:
    '''
    arcpy.SetProgressorLabel(message)
    arcpy.AddMessage(message)


def days(start, end):
    '''
    Generator that returns a datetime for tne next day each iteration
    :param start: datetime.date of start day
    :param end: datetime.date of last day
    :return: datetime.date of next day
    '''
    curr = start
    while curr <= end:
        yield curr
        curr = curr + datetime.timedelta(1)
        if curr.month == 2 and curr.day == 29:
            curr = curr + datetime.timedelta(1)

def import_toolboxes():
    tool_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    arcpy.ImportToolbox(os.path.join(tool_path,'FetchData.pyt'))
    arcpy.ImportToolbox(os.path.join(tool_path,'Misc.pyt'))
    arcpy.ImportToolbox(os.path.join(tool_path,'WaterBudget.pyt'))

def round_raster(raster):
    return arcpy.sa.RoundDown(raster * 100.0) / 100.0

def check_missing(project_folder, prefix, start_year, end_year, title, error):
    files = [os.path.join(project_folder, prefix, '%s_%d_%02d.tif' %(prefix, ym[0], ym[1])) for ym in
                 itertools.product(range(start_year, end_year + 1), range(1, 13))]
    exists = map(arcpy.Exists, files)
    if not all(exists):
        missing = [f[0] for f in zip(files, exists) if not f[1]]
        raise error('Missing %s Grids: %s' %(title, missing))

http_error = {
    100: ('Continue', 'Request received, please continue'),
    101: ('Switching Protocols',
          'Switching to new protocol; obey Upgrade header'),

    200: ('OK', 'Request fulfilled, document follows'),
    201: ('Created', 'Document created, URL follows'),
    202: ('Accepted',
          'Request accepted, processing continues off-line'),
    203: ('Non-Authoritative Information', 'Request fulfilled from cache'),
    204: ('No Content', 'Request fulfilled, nothing follows'),
    205: ('Reset Content', 'Clear input form for further input.'),
    206: ('Partial Content', 'Partial content follows.'),

    300: ('Multiple Choices',
          'Object has several resources -- see URI list'),
    301: ('Moved Permanently', 'Object moved permanently -- see URI list'),
    302: ('Found', 'Object moved temporarily -- see URI list'),
    303: ('See Other', 'Object moved -- see Method and URL list'),
    304: ('Not Modified',
          'Document has not changed since given time'),
    305: ('Use Proxy',
          'You must use proxy specified in Location to access this '
          'resource.'),
    307: ('Temporary Redirect',
          'Object moved temporarily -- see URI list'),

    400: ('Bad Request',
          'Bad request syntax or unsupported method'),
    401: ('Unauthorized',
          'No permission -- see authorization schemes'),
    402: ('Payment Required',
          'No payment -- see charging schemes'),
    403: ('Forbidden',
          'Request forbidden -- authorization will not help'),
    404: ('Not Found', 'Nothing matches the given URI'),
    405: ('Method Not Allowed',
          'Specified method is invalid for this server.'),
    406: ('Not Acceptable', 'URI not available in preferred format.'),
    407: ('Proxy Authentication Required', 'You must authenticate with '
                                           'this proxy before proceeding.'),
    408: ('Request Timeout', 'Request timed out; try again later.'),
    409: ('Conflict', 'Request conflict.'),
    410: ('Gone',
          'URI no longer exists and has been permanently removed.'),
    411: ('Length Required', 'Client must specify Content-Length.'),
    412: ('Precondition Failed', 'Precondition in headers is false.'),
    413: ('Request Entity Too Large', 'Entity is too large.'),
    414: ('Request-URI Too Long', 'URI is too long.'),
    415: ('Unsupported Media Type', 'Entity body in unsupported format.'),
    416: ('Requested Range Not Satisfiable',
          'Cannot satisfy request range.'),
    417: ('Expectation Failed',
          'Expect condition could not be satisfied.'),

    500: ('Internal Server Error', 'Server got itself in trouble'),
    501: ('Not Implemented',
          'Server does not support this operation'),
    502: ('Bad Gateway', 'Invalid responses from another server/proxy.'),
    503: ('Service Unavailable',
          'The server cannot process the request due to a high load'),
    504: ('Gateway Timeout',
          'The gateway server did not receive a timely response'),
    505: ('HTTP Version Not Supported', 'Cannot fulfill request.'),
    429: ('Too many requests','')
}
